@@ -12,6 +12,7 @@
   private
   public :: allocate_loopfun,getloop
   public :: aloopfun,bloopfun,b1loopfun,b11loopfun,cloopfun,dloopfun
+  integer, private :: ierr
 !
 ! variables for the 1-point sector:
 !
@@ -49,53 +50,42 @@
   end subroutine allocate_loopfun
 !
   subroutine allocate_loopfuna
-   integer :: nprop
-   nprop= dmns
-   allocate (aloopfun(0:2,2**(nprop-1)), stat=ierr)
+   allocate (aloopfun(0:2,dmns_a), stat=ierr)
    aloopfun= 0.d0
   end subroutine allocate_loopfuna
 !
   subroutine allocate_loopfunb
-   integer :: nprop
-   nprop= dmns
-   allocate (  bloopfun(0:2,2**(nprop-1)+2**(nprop-2)), stat=ierr)
-   allocate ( b1loopfun(0:2,2**(nprop-1)+2**(nprop-2)), stat=ierr)
-   allocate (b11loopfun(0:2,2**(nprop-1)+2**(nprop-2)), stat=ierr)
+   allocate (  bloopfun(0:2,dmns_b), stat=ierr)
+   allocate ( b1loopfun(0:2,dmns_b), stat=ierr)
+   allocate (b11loopfun(0:2,dmns_b), stat=ierr)
      bloopfun= 0.d0 
     b1loopfun= 0.d0
    b11loopfun= 0.d0
   end subroutine allocate_loopfunb
 !
   subroutine allocate_loopfunc
-   integer :: nprop
-   nprop= dmns
-   allocate (cloopfun(0:2,2**(nprop-1)+2**(nprop-2)+2**(nprop-3)), stat=ierr)
+   allocate (cloopfun(0:2,dmns_c), stat=ierr)
    cloopfun= 0.d0
   end subroutine allocate_loopfunc
 !
   subroutine allocate_loopfund
-   integer :: nprop
-   nprop= dmns
-   allocate (dloopfun(0:2,2**(nprop-1)+2**(nprop-2)+2**(nprop-3) &
-                                      +2**(nprop-4)), stat=ierr)
+   allocate (dloopfun(0:2,dmns_d), stat=ierr)
    dloopfun= 0.d0
   end subroutine allocate_loopfund
 !
   subroutine getloop(nprop)
    integer, intent(in) :: nprop
-   if (nprop.ge.1) call getaloop(nprop)
-   if (nprop.ge.2) call getbloop(nprop)
-   if (nprop.ge.3) call getcloop(nprop)
-   if (nprop.ge.4) call getdloop(nprop)
+   if (nprop.ge.1) call getaloop
+   if (nprop.ge.2) call getbloop
+   if (nprop.ge.3) call getcloop
+   if (nprop.ge.4) call getdloop
   end subroutine getloop
 !
-  subroutine getaloop(number_propagators)
+  subroutine getaloop
 !
 !  A 1-loop scalar function
 !
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_dpc.h'
     , dimension(0:2) :: value
    include 'cts_dpr.h'    
@@ -104,18 +94,16 @@
     :: cm12
    include 'cts_dpc.h' 
     :: a0,qlI1
-   np= number_propagators
-   do i= 1,nbn1(np)
-    ib= mbn1(np,i)
+   do ib= 1,dmns_1
     if     (scaloop.eq.1) then
 !     aloopfun(0,ib)= a0(m12)
      stop 'value of scaloop not implemented'
     elseif (scaloop.eq.2) then
-     cm12= den(bn1(np,1,i))%m2
+     cm12= den(bbn1(1,ib))%m2
      call olo(value,cm12)
      aloopfun(:,ib)= value(:) 
     elseif (scaloop.eq.3) then
-     m12= dreal(den(bn1(np,1,i))%m2)
+     m12= dreal(den(bbn1(1,ib))%m2)
      aloopfun(2,ib)= qlI1(m12,musq,-2)     
      aloopfun(1,ib)= qlI1(m12,musq,-1)     
      aloopfun(0,ib)= qlI1(m12,musq,0)     
@@ -125,14 +113,12 @@
    enddo
   end subroutine getaloop
 !
-  subroutine getbloop(number_propagators)
+  subroutine getbloop
 !
 !  B,B1 and B11 1-loop scalar functions
 !
    use tensor_operations  
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_dpr.h' 
     :: k12
    include 'cts_dpr.h'
@@ -147,10 +133,8 @@
     :: cm12,cm22
    include 'cts_dpc.h' 
     :: b0,b1,b11,qlI2
-   np= number_propagators
-   do i= 1,nbn2(np)
-    ib= mbn2(np,i)
-    k1= den(bn2(np,2,i))%p-den(bn2(np,1,i))%p
+   do ib= 1,dmns_2
+    k1= den(bbn2(2,ib))%p-den(bbn2(1,ib))%p
     call contr(k1,k1,k12)
 !!    if (dabs(k12/roots/roots).lt.1.d-8) k12= 0.d0
     if     (scaloop.eq.1) then
@@ -159,15 +143,15 @@
 !     b11loopfun(0,ib)= b11(k12,m12,m22)
      stop 'value of scaloop not implemented'
     elseif (scaloop.eq.2) then
-     cm12= den(bn2(np,1,i))%m2
-     cm22= den(bn2(np,2,i))%m2
+     cm12= den(bbn2(1,ib))%m2
+     cm22= den(bbn2(2,ib))%m2
      call olo(valb11,valb00,valb1,valb0,k12,cm12,cm22)
      bloopfun(:,ib)  =  valb0(:)
      b1loopfun(:,ib) =  valb1(:)
      b11loopfun(:,ib)=  valb11(:)
     elseif (scaloop.eq.3) then
-     m12= dreal(den(bn2(np,1,i))%m2)
-     m22= dreal(den(bn2(np,2,i))%m2)
+     m12= dreal(den(bbn2(1,ib))%m2)
+     m22= dreal(den(bbn2(2,ib))%m2)
      bloopfun(2,ib)= qlI2(k12,m12,m22,musq,-2)     
      bloopfun(1,ib)= qlI2(k12,m12,m22,musq,-1)     
      bloopfun(0,ib)= qlI2(k12,m12,m22,musq,0)     
@@ -180,14 +164,12 @@
    enddo
   end subroutine getbloop
 !
-  subroutine getcloop(number_propagators)
+  subroutine getcloop
 !
 !  C 1-loop scalar function
 !
    use tensor_operations  
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_dpr.h' 
     :: k12,k22,k32
    include 'cts_dpr.h'
@@ -200,12 +182,10 @@
     :: cm12,cm22,cm32
    include 'cts_dpc.h' 
     :: c0,qlI3
-   np= number_propagators
-   do i= 1,nbn3(np)
-    ib= mbn3(np,i)
-    k1= den(bn3(np,2,i))%p-den(bn3(np,1,i))%p
-    k2= den(bn3(np,3,i))%p-den(bn3(np,2,i))%p
-    k3= den(bn3(np,3,i))%p-den(bn3(np,1,i))%p
+   do ib= 1,dmns_3
+    k1= den(bbn3(2,ib))%p-den(bbn3(1,ib))%p
+    k2= den(bbn3(3,ib))%p-den(bbn3(2,ib))%p
+    k3= den(bbn3(3,ib))%p-den(bbn3(1,ib))%p
     call contr(k1,k1,k12)
     call contr(k2,k2,k22)
     call contr(k3,k3,k32)
@@ -216,15 +196,15 @@
 !     cloopfun(0,ib)  =  c0(k12,k22,k32,m12,m22,m32) 
      stop 'value of scaloop not implemented'
     elseif (scaloop.eq.2) then
-     cm12= den(bn3(np,1,i))%m2
-     cm22= den(bn3(np,2,i))%m2
-     cm32= den(bn3(np,3,i))%m2
+     cm12= den(bbn3(1,ib))%m2
+     cm22= den(bbn3(2,ib))%m2
+     cm32= den(bbn3(3,ib))%m2
      call olo(value,k12,k22,k32,cm12,cm22,cm32)
      cloopfun(:,ib)  =        value(:) 
     elseif (scaloop.eq.3) then
-     m12= dreal(den(bn3(np,1,i))%m2)
-     m22= dreal(den(bn3(np,2,i))%m2)
-     m32= dreal(den(bn3(np,3,i))%m2)
+     m12= dreal(den(bbn3(1,ib))%m2)
+     m22= dreal(den(bbn3(2,ib))%m2)
+     m32= dreal(den(bbn3(3,ib))%m2)
      cloopfun(2,ib)= qlI3(k12,k22,k32,m12,m22,m32,musq,-2)     
      cloopfun(1,ib)= qlI3(k12,k22,k32,m12,m22,m32,musq,-1)     
      cloopfun(0,ib)= qlI3(k12,k22,k32,m12,m22,m32,musq,0)     
@@ -234,14 +214,12 @@
    enddo
   end subroutine getcloop
 !
-  subroutine getdloop(number_propagators)
+  subroutine getdloop
 !
 !  D 1-loop scalar function
 !
    use tensor_operations  
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_dpr.h' 
     :: k12,k22,k32,k42,k122,k232
    include 'cts_dpr.h'
@@ -254,15 +232,13 @@
     :: cm12,cm22,cm32,cm42
    include 'cts_dpc.h' 
     :: d0,qlI4
-   np= number_propagators
-   do i= 1,nbn4(np)
-    ib= mbn4(np,i)
-    k1 = den(bn4(np,2,i))%p-den(bn4(np,1,i))%p
-    k2 = den(bn4(np,3,i))%p-den(bn4(np,2,i))%p
-    k3 = den(bn4(np,4,i))%p-den(bn4(np,3,i))%p
-    k4 = den(bn4(np,4,i))%p-den(bn4(np,1,i))%p
-    p12= den(bn4(np,3,i))%p-den(bn4(np,1,i))%p
-    p23= den(bn4(np,4,i))%p-den(bn4(np,2,i))%p
+   do ib= 1,dmns_4
+    k1 = den(bbn4(2,ib))%p-den(bbn4(1,ib))%p
+    k2 = den(bbn4(3,ib))%p-den(bbn4(2,ib))%p
+    k3 = den(bbn4(4,ib))%p-den(bbn4(3,ib))%p
+    k4 = den(bbn4(4,ib))%p-den(bbn4(1,ib))%p
+    p12= den(bbn4(3,ib))%p-den(bbn4(1,ib))%p
+    p23= den(bbn4(4,ib))%p-den(bbn4(2,ib))%p
     call contr(k1 ,k1 ,k12)
     call contr(k2 ,k2 ,k22)
     call contr(k3 ,k3 ,k32)
@@ -279,17 +255,17 @@
 !     dloopfun(0,ib)  = d0(k12,k22,k32,k42,k122,k232,m12,m22,m32,m42)
      stop 'value of scaloop not implemented'
     elseif (scaloop.eq.2) then
-     cm12= den(bn4(np,1,i))%m2
-     cm22= den(bn4(np,2,i))%m2
-     cm32= den(bn4(np,3,i))%m2
-     cm42= den(bn4(np,4,i))%m2
+     cm12= den(bbn4(1,ib))%m2
+     cm22= den(bbn4(2,ib))%m2
+     cm32= den(bbn4(3,ib))%m2
+     cm42= den(bbn4(4,ib))%m2
      call olo(value,k12,k22,k32,k42,k122,k232,cm12,cm22,cm32,cm42)
      dloopfun(:,ib)  =        value(:) 
     elseif (scaloop.eq.3) then
-     m12= dreal(den(bn4(np,1,i))%m2)
-     m22= dreal(den(bn4(np,2,i))%m2)
-     m32= dreal(den(bn4(np,3,i))%m2)
-     m42= dreal(den(bn4(np,4,i))%m2)
+     m12= dreal(den(bbn4(1,ib))%m2)
+     m22= dreal(den(bbn4(2,ib))%m2)
+     m32= dreal(den(bbn4(3,ib))%m2)
+     m42= dreal(den(bbn4(4,ib))%m2)
      dloopfun(2,ib)= qlI4(k12,k22,k32,k42,k122,k232,m12,m22,m32,m42,musq,-2)   
      dloopfun(1,ib)= qlI4(k12,k22,k32,k42,k122,k232,m12,m22,m32,m42,musq,-1)   
      dloopfun(0,ib)= qlI4(k12,k22,k32,k42,k122,k232,m12,m22,m32,m42,musq,0)   
@@ -307,6 +283,7 @@
 !
 !
  module mp_loopfunctions
+  include 'cts_mprec.h'
   use denominators
   use scale
   use dimensions
@@ -317,7 +294,7 @@
   public :: mp_aloopfun,mp_bloopfun,mp_b1loopfun,mp_b11loopfun,mp_cloopfun,mp_dloopfun
   include 'cts_mpc.h'
    , private :: mp_czero
-  integer, private :: ic
+  integer, private :: ic,ierr
 !
 ! variables for the 1-point sector:
 !
@@ -357,53 +334,42 @@
   end subroutine allocate_mp_loopfun
 !
   subroutine allocate_mp_loopfuna
-   integer :: nprop
-   nprop= dmns
-   allocate (mp_aloopfun(0:2,2**(nprop-1)), stat=ierr)
+   allocate (mp_aloopfun(0:2,dmns_a), stat=ierr)
    mp_aloopfun= mp_czero
   end subroutine allocate_mp_loopfuna
 !
   subroutine allocate_mp_loopfunb
-   integer :: nprop
-   nprop= dmns
-   allocate (  mp_bloopfun(0:2,2**(nprop-1)+2**(nprop-2)), stat=ierr)
-   allocate ( mp_b1loopfun(0:2,2**(nprop-1)+2**(nprop-2)), stat=ierr)
-   allocate (mp_b11loopfun(0:2,2**(nprop-1)+2**(nprop-2)), stat=ierr)
+   allocate (  mp_bloopfun(0:2,dmns_b), stat=ierr)
+   allocate ( mp_b1loopfun(0:2,dmns_b), stat=ierr)
+   allocate (mp_b11loopfun(0:2,dmns_b), stat=ierr)
      mp_bloopfun= mp_czero 
     mp_b1loopfun= mp_czero
    mp_b11loopfun= mp_czero
   end subroutine allocate_mp_loopfunb
 !
   subroutine allocate_mp_loopfunc
-   integer :: nprop
-   nprop= dmns
-   allocate (mp_cloopfun(0:2,2**(nprop-1)+2**(nprop-2)+2**(nprop-3)), stat=ierr)
+   allocate (mp_cloopfun(0:2,dmns_c), stat=ierr)
    mp_cloopfun= mp_czero
   end subroutine allocate_mp_loopfunc
 !
   subroutine allocate_mp_loopfund
-   integer :: nprop
-   nprop= dmns
-   allocate (mp_dloopfun(0:2,2**(nprop-1)+2**(nprop-2)+2**(nprop-3) &
-                                         +2**(nprop-4)), stat=ierr)
+   allocate (mp_dloopfun(0:2,dmns_d), stat=ierr)
    mp_dloopfun= mp_czero
   end subroutine allocate_mp_loopfund
 !
   subroutine get_mp_loop(nprop)
    integer, intent(in) :: nprop
-   if (nprop.ge.1) call get_mp_aloop(nprop)
-   if (nprop.ge.2) call get_mp_bloop(nprop)
-   if (nprop.ge.3) call get_mp_cloop(nprop)
-   if (nprop.ge.4) call get_mp_dloop(nprop)
+   if (nprop.ge.1) call get_mp_aloop
+   if (nprop.ge.2) call get_mp_bloop
+   if (nprop.ge.3) call get_mp_cloop
+   if (nprop.ge.4) call get_mp_dloop
   end subroutine get_mp_loop
 !
-  subroutine get_mp_aloop(number_propagators)
+  subroutine get_mp_aloop
 !
 !  A 1-loop scalar function
 !
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_mpc.h'
     , dimension(0:2) :: value
    include 'cts_mpr.h'    
@@ -414,15 +380,13 @@
     :: qlI1
    include 'cts_dpr.h'    
     :: dm12
-   np= number_propagators
-   do i= 1,nbn1(np)
-    ib= mbn1(np,i)
+   do ib= 1,dmns_1
     if     (scaloop.eq.2) then
-     cm12= mp_den(bn1(np,1,i))%m2
+     cm12= mp_den(bbn1(1,ib))%m2
      call olo(value,cm12)
      mp_aloopfun(:,ib)= value(:) 
     elseif (scaloop.eq.3) then
-     dm12= mp_den(bn1(np,1,i))%m2
+     dm12= mp_den(bbn1(1,ib))%m2
      mp_aloopfun(2,ib)= qlI1(dm12,musq,-2)     
      mp_aloopfun(1,ib)= qlI1(dm12,musq,-1)     
      mp_aloopfun(0,ib)= qlI1(dm12,musq,0)     
@@ -432,14 +396,12 @@
    enddo
   end subroutine get_mp_aloop
 !
-  subroutine get_mp_bloop(number_propagators)
+  subroutine get_mp_bloop
 !
 !  B,B1 and B11 1-loop scalar functions
 !
    use tensor_operations  
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_mpr.h' 
     :: k12
    include 'cts_mpr.h'
@@ -460,25 +422,23 @@
     :: dm12,dm22
    include 'cts_dpr.h' 
     :: dk12
-   np= number_propagators
-   do i= 1,nbn2(np)
-    ib= mbn2(np,i)
+   do ib= 1,dmns_2
     do ic= 0,3
-     k1(ic)= mp_den(bn2(np,2,i))%p(ic) &
-            -mp_den(bn2(np,1,i))%p(ic)
+     k1(ic)= mp_den(bbn2(2,ib))%p(ic) &
+            -mp_den(bbn2(1,ib))%p(ic)
     enddo
     call contr(k1,k1,k12)
 !!    if (dabs(k12/roots/roots).lt.1.d-8) k12= 0.d0
     if     (scaloop.eq.2) then
-     cm12= mp_den(bn2(np,1,i))%m2
-     cm22= mp_den(bn2(np,2,i))%m2
+     cm12= mp_den(bbn2(1,ib))%m2
+     cm22= mp_den(bbn2(2,ib))%m2
      call olo(valb11,valb00,valb1,valb0,k12,cm12,cm22)
      mp_bloopfun(:,ib)  =  valb0(:)
      mp_b1loopfun(:,ib) =  valb1(:)
      mp_b11loopfun(:,ib)=  valb11(:)
     elseif (scaloop.eq.3) then
-     dm12= mp_den(bn2(np,1,i))%m2
-     dm22= mp_den(bn2(np,2,i))%m2
+     dm12= mp_den(bbn2(1,ib))%m2
+     dm22= mp_den(bbn2(2,ib))%m2
      dk12= k12 
      mp_bloopfun(2,ib)= qlI2(dk12,dm12,dm22,musq,-2)     
      mp_bloopfun(1,ib)= qlI2(dk12,dm12,dm22,musq,-1)     
@@ -496,14 +456,12 @@
    enddo
   end subroutine get_mp_bloop
 !
-  subroutine get_mp_cloop(number_propagators)
+  subroutine get_mp_cloop
 !
 !  C 1-loop scalar function
 !
    use tensor_operations  
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_mpr.h' 
     :: k12,k22,k32
    include 'cts_mpr.h'
@@ -520,16 +478,14 @@
     :: dm12,dm22,dm32
    include 'cts_dpr.h' 
     :: dk12,dk22,dk32
-   np= number_propagators
-   do i= 1,nbn3(np)
-    ib= mbn3(np,i)
+   do ib= 1,dmns_3
     do ic= 0,3
-     k1(ic)= mp_den(bn3(np,2,i))%p(ic) &
-            -mp_den(bn3(np,1,i))%p(ic)
-     k2(ic)= mp_den(bn3(np,3,i))%p(ic) &
-            -mp_den(bn3(np,2,i))%p(ic)
-     k3(ic)= mp_den(bn3(np,3,i))%p(ic) &
-            -mp_den(bn3(np,1,i))%p(ic)
+     k1(ic)= mp_den(bbn3(2,ib))%p(ic) &
+            -mp_den(bbn3(1,ib))%p(ic)
+     k2(ic)= mp_den(bbn3(3,ib))%p(ic) &
+            -mp_den(bbn3(2,ib))%p(ic)
+     k3(ic)= mp_den(bbn3(3,ib))%p(ic) &
+            -mp_den(bbn3(1,ib))%p(ic)
     enddo
     call contr(k1,k1,k12)
     call contr(k2,k2,k22)
@@ -538,15 +494,15 @@
 !!    if (dabs(k22/roots/roots).lt.1.d-8) k22= 0.d0
 !!    if (dabs(k32/roots/roots).lt.1.d-8) k32= 0.d0
     if     (scaloop.eq.2) then
-     cm12= mp_den(bn3(np,1,i))%m2
-     cm22= mp_den(bn3(np,2,i))%m2
-     cm32= mp_den(bn3(np,3,i))%m2
+     cm12= mp_den(bbn3(1,ib))%m2
+     cm22= mp_den(bbn3(2,ib))%m2
+     cm32= mp_den(bbn3(3,ib))%m2
      call olo(value,k12,k22,k32,cm12,cm22,cm32)
      mp_cloopfun(:,ib)  =        value(:) 
     elseif (scaloop.eq.3) then
-     dm12= mp_den(bn3(np,1,i))%m2
-     dm22= mp_den(bn3(np,2,i))%m2
-     dm32= mp_den(bn3(np,3,i))%m2
+     dm12= mp_den(bbn3(1,ib))%m2
+     dm22= mp_den(bbn3(2,ib))%m2
+     dm32= mp_den(bbn3(3,ib))%m2
      dk12= k12
      dk22= k22
      dk32= k32
@@ -559,14 +515,12 @@
    enddo
   end subroutine get_mp_cloop
 !
-  subroutine get_mp_dloop(number_propagators)
+  subroutine get_mp_dloop
 !
 !  D 1-loop scalar function
 !
    use tensor_operations  
-   integer, intent(in) :: number_propagators
-   integer :: np
-   integer :: i,ib
+   integer :: ib
    include 'cts_mpr.h' 
     :: k12,k22,k32,k42,k122,k232
    include 'cts_mpr.h'
@@ -583,22 +537,20 @@
     :: dm12,dm22,dm32,dm42
    include 'cts_dpr.h' 
     :: dk12,dk22,dk32,dk42,dk122,dk232
-   np= number_propagators
-   do i= 1,nbn4(np)
-    ib= mbn4(np,i)
+   do ib= 1,dmns_4
     do ic= 0,3
-     k1(ic) = mp_den(bn4(np,2,i))%p(ic) &
-             -mp_den(bn4(np,1,i))%p(ic)
-     k2(ic) = mp_den(bn4(np,3,i))%p(ic) &
-             -mp_den(bn4(np,2,i))%p(ic)
-     k3(ic) = mp_den(bn4(np,4,i))%p(ic) &
-             -mp_den(bn4(np,3,i))%p(ic)
-     k4(ic) = mp_den(bn4(np,4,i))%p(ic) &
-             -mp_den(bn4(np,1,i))%p(ic)
-     p12(ic)= mp_den(bn4(np,3,i))%p(ic) &
-             -mp_den(bn4(np,1,i))%p(ic)
-     p23(ic)= mp_den(bn4(np,4,i))%p(ic) &
-             -mp_den(bn4(np,2,i))%p(ic)
+     k1(ic) = mp_den(bbn4(2,ib))%p(ic) &
+             -mp_den(bbn4(1,ib))%p(ic)
+     k2(ic) = mp_den(bbn4(3,ib))%p(ic) &
+             -mp_den(bbn4(2,ib))%p(ic)
+     k3(ic) = mp_den(bbn4(4,ib))%p(ic) &
+             -mp_den(bbn4(3,ib))%p(ic)
+     k4(ic) = mp_den(bbn4(4,ib))%p(ic) &
+             -mp_den(bbn4(1,ib))%p(ic)
+     p12(ic)= mp_den(bbn4(3,ib))%p(ic) &
+             -mp_den(bbn4(1,ib))%p(ic)
+     p23(ic)= mp_den(bbn4(4,ib))%p(ic) &
+             -mp_den(bbn4(2,ib))%p(ic)
     enddo
     call contr(k1 ,k1 ,k12)
     call contr(k2 ,k2 ,k22)
@@ -613,17 +565,17 @@
 !!    if (dabs(k122/roots/roots).lt.1.d-8) k122= 0.d0
 !!    if (dabs(k232/roots/roots).lt.1.d-8) k232= 0.d0
     if     (scaloop.eq.2) then
-     cm12= mp_den(bn4(np,1,i))%m2
-     cm22= mp_den(bn4(np,2,i))%m2
-     cm32= mp_den(bn4(np,3,i))%m2
-     cm42= mp_den(bn4(np,4,i))%m2
+     cm12= mp_den(bbn4(1,ib))%m2
+     cm22= mp_den(bbn4(2,ib))%m2
+     cm32= mp_den(bbn4(3,ib))%m2
+     cm42= mp_den(bbn4(4,ib))%m2
      call olo(value,k12,k22,k32,k42,k122,k232,cm12,cm22,cm32,cm42)
      mp_dloopfun(:,ib)  =        value(:) 
     elseif (scaloop.eq.3) then
-     dm12= mp_den(bn4(np,1,i))%m2
-     dm22= mp_den(bn4(np,2,i))%m2
-     dm32= mp_den(bn4(np,3,i))%m2
-     dm42= mp_den(bn4(np,4,i))%m2
+     dm12= mp_den(bbn4(1,ib))%m2
+     dm22= mp_den(bbn4(2,ib))%m2
+     dm32= mp_den(bbn4(3,ib))%m2
+     dm42= mp_den(bbn4(4,ib))%m2
      dk12 = k12
      dk22 = k22
      dk32 = k32
